@@ -1,4 +1,4 @@
-`print.gpd` <-
+print.gpd <-
 function( x , digits=max(3, getOption("digits") - 3), ... ){
 	cat( "Call: " )
 	print( x$call, ... )
